package slgui.base;

/** This should uniquely identify an installed/installable version of an application on the system,
 * but without any installation-specific details such as file paths. This can be used for checking versions,
 * loading default configurations, for generating default installation paths, (in the future) sending messages
 * between apps, and so on. For the most part though, it's just used internally wherever the app name is
 * needed. It also helps to ensure that app release ids are reasonably consistent
 * 
 * @author Zak Fenton
 *
 */
public final class MediaAppId {
	public final String marketId;
	public final String developerId;
	public final String releaseId;
	public final int majorVersion;
	public final int minorVersion;
	public final String patchId;

	public MediaAppId(String marketId, String developerId, String releaseId, int majorVersion, int minorVersion, String patchId) {
		this.marketId = checkString(marketId);
		this.developerId = checkString(developerId);
		this.releaseId = checkString(releaseId);
		this.majorVersion = majorVersion;
		this.minorVersion = minorVersion;
		this.patchId = checkString(patchId);
	}
	
	public boolean stringFails(String str) {
		if (str.length() < 1) {
			return true;
		} else {
			return str.contains("/") || str.contains("\\") || str.contains("-") || str.contains("*") || str.contains(".");
		}
	}
	
	public String checkString(String str) {
		if (stringFails(str)) {
			throw new Error("The string '" + str + "' contains characters forbidden in app ids");
		}
		return str;
	}

}
