package slgui.base;

import java.util.HashMap;

public class MediaInputState {
	private HashMap<String, MediaPointerState> pointerstates = new HashMap<String, MediaPointerState>();
	private HashMap<String, Boolean> keystates = new HashMap<String, Boolean>();
	double lastX, lastY;

	public MediaInputState() {
		// TODO Auto-generated constructor stub
	}
	
	public synchronized void resetKeypad() {
		keystates = new HashMap<String, Boolean>();
	}
	
	public synchronized void resetPointers() {
		pointerstates = new HashMap<String, MediaPointerState>();
	}
	
	public synchronized MediaPointerState setPointer(String id, double x, double y, int nbuttons, MediaPointerButton... buttons) {
		if (nbuttons < 0) {
			pointerstates.remove(id);
			return null;
		}
		if (!pointerstates.containsKey(id)) {
			pointerstates.put(id, new MediaPointerState());
		}
		MediaPointerState s = pointerstates.get(id);
		s.reset(id);
		s.setCoordinates(x, y);
		lastX = x;
		lastY = y;
		s.setButtons(nbuttons, 0);
		for (int i = 0; i < buttons.length; i++) {
			s.setButton(buttons[i], true);
		}
		return s;
	}
	
	public boolean hoverWithin(double x, double y, double width, double height) {
		MediaPointerState[] p = currentPointers();
		for (int i = 0; i < p.length; i++) {
			if (p[i].hoverWithin(x,y,width,height)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean downWithin(MediaPointerButton b, double x, double y, double width, double height) {
		MediaPointerState[] p = currentPointers();
		for (int i = 0; i < p.length; i++) {
			if (p[i].hoverWithin(x,y,width,height) && p[i].buttonDown(b)) {
				return true;
			}
		}
		return false;
	}
	
	public synchronized MediaPointerState[] currentPointers() {
		return pointerstates.values().toArray(new MediaPointerState[0]);
	}
	
	public synchronized void setKeypad(String value, boolean down) {
		keystates.put(value, down);
	}
	
	public synchronized boolean keypadDown(String value) {
		if (keystates.containsKey(value)) {
			return keystates.get(value);
		} else {
			return false;
		}
	}

	public double lastPointerX() {
		return lastX;
	}

	public double lastPointerY() {
		return lastY;
	}

}
