package slgui.base;

/* A MediaPainter object represents a minimalist rendering target, i.e. something
 * which in theory could be implemented on (robotic) pen & paper, but which should
 * be equally convenient for on-screen use. The design focuses on pen-like operations
 * for this reason, and does not in itself provide a heap of convenience functions
 */
public abstract class MediaPainter {
	/** Returns the given x value, except "snapped" to the centre of the nearest pixel (if applicable). */
	public double snapX(double x) {
		return x;
	}
	/** Returns the given y value, except "snapped" to the centre of the nearest pixel (if applicable). */
	public double snapY(double y) {
		return y;
	}
	/** Draws a line with the given (ARGB32) pixel value and radius. */
	public abstract void line(double x1, double y1, double x2, double y2, int pixel, double radius);
	/** Fills a simple box with the given (ARGB32) pixel value. */
	public abstract void box(double x, double y, double width, double height, int pixel);
}
