package slgui.base;

public class MediaPointerState {
	String pointerId;
	double x, y;
	int buttonFlags;
	int nbuttons;

	public MediaPointerState() {
		reset(null);
	}
	
	public synchronized void reset(String pointerId) {
		this.pointerId = pointerId;
		x = -1;
		y = -1;
		nbuttons = -1;
		buttonFlags = 0;
	}

	public synchronized void setCoordinates(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	public synchronized MediaPoint getCoordinates() {
		return new MediaPoint(x, y);
	}
	
	public synchronized void setButtons(int numberOfButtons, int buttonFlags) {
		this.nbuttons = numberOfButtons;
		this.buttonFlags = buttonFlags;
	}
	
	public synchronized boolean buttonDown(MediaPointerButton b) {
		return buttonExists(b) && ((buttonFlags & (1 << b.ordinal())) != 0);
	}
	
	public synchronized void setButton(MediaPointerButton b, boolean down) {
		if (b != MediaPointerButton.MAIN && nbuttons < b.ordinal()+1) {
			nbuttons = b.ordinal()+1;
		}
		if (down) {
			buttonFlags |= (1 << b.ordinal());
		} else {
			buttonFlags &= ~(1 << b.ordinal());
		}
	}
	
	public synchronized boolean isTouch() {
		return nbuttons == 0;
	}
	
	public boolean isValid() {
		return pointerId != null;
	}
	
	public synchronized boolean buttonExists(MediaPointerButton b) {
		if (b == MediaPointerButton.MAIN) {
			return true;
		} else {
			return nbuttons >= b.ordinal()+1;
		}
	}

	public boolean hoverWithin(double x2, double y2, double width, double height) {
		if (x >= x2 && x <= x2 + width && y >= y2 && y <= y2 + height) {
			return true;
		} else {
			return false;
		}
	}
}
