package slgui.gfx;

/** A TranslateTransformer only adds a "translation" offset to the x/y coordinates. It has
 * no effect on other (width/height-related) coordinates.
 * 
 * @author Zak Fenton
 *
 */
public final class TranslateTransformer extends Transformer {
	public final double tx;
	public final double ty;
	
	public TranslateTransformer(double tx, double ty) {
		this.tx = tx;
		this.ty = ty;
	}

	@Override
	public double x(double x) {
		return tx + x;
	}
	
	@Override
	public double y(double y) {
		return tx + y;
	}

	@Override
	public double unX(double x) {
		return x - tx;
	}
	
	@Override
	public double unY(double y) {
		return y - ty;
	}
	
	@Override
	public Transformer combinedTransformer(Transformer other) {
		if (other instanceof TranslateTransformer) {
			TranslateTransformer o = (TranslateTransformer) other;
			return new TranslateTransformer(this.tx + o.tx, this.ty + o.ty);
		} else {
			return super.combinedTransformer(other);
		}
	}
}
