package slgui.space;

import java.util.ArrayList;
import java.util.Date;

import slgui.base.MediaBox;
import slgui.base.MediaFont;
import slgui.base.MediaInputState;
import slgui.base.MediaPainter;
import slgui.base.MediaPointerButton;
import slgui.themes.KeypadType;
import slgui.themes.PanelMode;
import slgui.themes.StatusMode;
import slgui.themes.Theme;

public class SpaceState {
	private SpaceDelegate delegate;
	private boolean dirty = true;
	private ArrayList<WindowState> windows = new ArrayList<WindowState>();
	WindowState dragging;
	double dragX, dragY;
	KeypadType keypad;
	KeypadType nextKeypad; /* This value is eventually assigned to "keypad", after animation. */
	MediaFont font = new MediaFont(2);
	
	private double x, y, width, height;
	private MediaInputState input = new MediaInputState();

	public SpaceState(SpaceDelegate delegate) {
		this.delegate = delegate;
		delegate.attachSpace(this);
	}

	public boolean isDirty() {
		return dirty;
	}
	
	public void markDirty(boolean value) {
		this.dirty = value;
	}
	
	public void markDirty() {
		markDirty(true);
	}
	
	public void setInputState(MediaInputState input) {
		this.input = input;
		
		if (input.downWithin(MediaPointerButton.MAIN, x, y, width, height)) {
			if (dragging != null) {
				if (dragX > 0) {
					System.out.println("Continguing drag");
					MediaBox coords = dragging.realCoordinates();
					dragging.setCoordinates(input.lastPointerX() - dragX, input.lastPointerY() - dragY, coords.width, coords.height);
				} else {
					System.out.println("Continguing resize");
					MediaBox coords = dragging.realCoordinates();
					dragging.setCoordinates(coords.x, coords.y, Math.max(100, input.lastPointerX() - coords.x), Math.max(100, input.lastPointerY() - coords.y));
				}
			} else {
			WindowState[] wins = currentWindows();
				for (int i = wins.length - 1; i >= 0; i--) {
					MediaBox coords = wins[i].realCoordinates();
					// TODO: The drag area should be dictated by the theme
					if (input.downWithin(MediaPointerButton.MAIN, coords.x, coords.y, coords.width, coords.height)) {
						if (input.downWithin(MediaPointerButton.MAIN, coords.x + 24, coords.y, coords.width-48, 24)) {
							System.out.println("Starting drag");
							dragging = wins[i];
							dragging.bringToFront();
							dragX = input.lastPointerX() - coords.x;
							dragY = input.lastPointerY() - coords.y;
							System.out.println("Dragx=" +dragX + "y=" + dragY);
						} else if (input.downWithin(MediaPointerButton.MAIN, coords.x + coords.width-16, coords.y + coords.height-16, 16, 16)) {
							System.out.println("Starting resize");
							dragging = wins[i];
							dragging.bringToFront();
							dragX = input.lastPointerX() - (coords.x + coords.width);
							dragY = input.lastPointerY() - (coords.y + coords.height);
							System.out.println("Dragx=" +dragX + "y=" + dragY);
						}
						break;
					}
				}
			}
		} else {
			dragging = null;
		}
	}
	
	public MediaInputState getInputState() {
		return input;
	}

	public synchronized WindowState newWindow(WindowDelegate delegate) {
		WindowState result = new WindowState(this, delegate);
		windows.add(result);
		markDirty();
		return result;
	}
	
	public synchronized void bringToFront(WindowState w) {
		int winidx = windows.indexOf(w);
		if (winidx < 0) {
			return;
		}
		ArrayList<WindowState> newWindows = new ArrayList<WindowState>(windows.size());
		for (int i = 0; i < windows.size(); i++) {
			if (i < winidx) {
				newWindows.add(windows.get(i));
			} else if (i == windows.size()-1) {
				newWindows.add(w);
			} else {
				newWindows.add(windows.get(i + 1));
			}
		}
		windows = newWindows;
	}
	
	public void setCoordinates(double x, double y, double width, double height) {
		if (x != this.x || y != this.y || width != this.width || height != this.height) {
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			markDirty();
		}
	}
	
	public synchronized WindowState[] currentWindows() {
		return windows.toArray(new WindowState[0]);
	}
	
	public void paint(MediaPainter p) {
		markDirty(false);
		p.line(10, 100, width-10, height-100, 0xFFFF0000, 10.0);
		p.line(width-100, 10, 100, height-10, 0xFFFF0000, 1.0);
		font.render(p, "Hello, world!", 100, 200, 0xFF50C0FF);
		new MediaFont(1).render(p, "Hello, world!", 100, 225, 0xFF50C0FF);
		new MediaFont(4).render(p, "Hello, world! 0123456789", 100, 250, 0xFF50C0FF);
		new MediaFont(4).render(p, "Hello... world?", 100, 300, 0xFF50C0FF);
		new MediaFont(2).render(p, "Sphinx of black quartz, judge my vow.", 100, 350, 0xFF50C0FF);
		new MediaFont(1).render(p, "Sphinx of black quartz, judge my vow. 0123456789", 100, 400, 0xFF50C0FF);
		
		Theme t = new Theme();
		double keyanim = Math.sin((System.currentTimeMillis()%3000)/1000.0);
		MediaBox content = t.desktopContentArea(p, 0,0, width, height, KeypadType.LATIN_LOWER, keyanim, StatusMode.TOP, PanelMode.BOTTOM);
		
		WindowState[] wins = currentWindows();
		double innerX = content.x;
		double innerY = content.y;
		double innerWidth = content.width;
		double innerHeight = content.height;
		for (int i = 0; i < wins.length; i++) {
			/* This will, if necessary, resize any windows to appropriate bounds. They may still
			 * exceed these bounds (or be entirely "off-screen"), but only if the user has some
			 * way of controlling the window.
			 */
			wins[i].realiseCoordinates(p, t, innerX, innerY, innerWidth, innerHeight);
		}
		for (int i = 0; i < wins.length; i++) {
			wins[i].paint(p, getInputState(), t);
		}
		t.paintDemoWindow(p, getInputState(), "A window", 300, 25, 250, 150);
		t.paintDemoDesktop(p, getInputState(), delegate.getStatus1(), delegate.getStatus2(), 0,0, width, height, KeypadType.LATIN_LOWER, keyanim, StatusMode.TOP, PanelMode.BOTTOM);
	}
}
