package slgui.space;

import slgui.base.MediaBox;
import slgui.base.MediaInputState;
import slgui.base.MediaPainter;
import slgui.themes.Theme;

public class WindowState {
	private SpaceState space;
	private WindowDelegate delegate;
	private boolean dirty = true;
	private boolean redraw = true;
	private double realX, realY, realWidth, realHeight;
	private double configuredX, configuredY, configuredWidth, configuredHeight;
	private String title;
	private SizingState sizing = SizingState.WINDOWED;

	WindowState(SpaceState space, WindowDelegate delegate) {
		this.space = space;
		this.delegate = delegate;
		title = "Untitled";
	}

	public boolean isDirty() {
		return dirty;
	}
	
	public void markDirty(boolean value) {
		if (value && !dirty) {
			space.markDirty();
		}
		this.dirty = value;
	}
	
	public void markDirty() {
		markDirty(true);
	}
	
	public boolean shouldRedraw() {
		return redraw;
	}
	
	public void markRedraw(boolean value) {
		this.redraw = value;
	}
	
	public void markRedraw() {
		markRedraw(true);
	}
	
	public void setSizingState(SizingState sizing) {
		if (sizing != this.sizing) { 
			this.sizing = sizing;
			markDirty();
			markRedraw();
		}
	}
	
	public SizingState getSizingState() {
		return this.sizing;
	}
	
	public synchronized void realiseCoordinates(MediaPainter p, Theme t, double minX, double minY, double maxWidth, double maxHeight) {
		double newX = Math.max(minX, configuredX);
		double newY = Math.max(minY, configuredY);
		double newWidth = configuredWidth; //Math.min(maxWidth, configuredWidth);
		double newHeight = configuredHeight; //Math.min(maxHeight, configuredHeight);

		if (newX > minX + maxWidth - 100) {
			newX = minX + maxWidth - 100;
		}
		if (newY > minY + maxHeight - 32) {
			newY = minY + maxHeight - 32;
		}
		
		if (newX != realX || newY != realY || newWidth != realWidth || newHeight != realHeight) {
			realX = newX;
			realY = newY;
			realWidth = newWidth;
			realHeight = newHeight;
			markRedraw();
			markDirty();
		}
	}
	
	public synchronized void setCoordinates(double x, double y, double width, double height) {
		if (x != configuredX || y != configuredY || width != configuredWidth || height != configuredHeight) {
			configuredX = x;
			configuredY = y;
			configuredWidth = width;
			configuredHeight = height;
			markRedraw();
			markDirty();
		}
	}
	
	public synchronized MediaBox realCoordinates() {
		return new MediaBox(realX, realY, realWidth, realHeight);
	}
	
	public void bringToFront() {
		space.bringToFront(this);
	}

	public void paint(MediaPainter p, MediaInputState input, Theme t) {
		t.paintDemoWindow(p, input, title, realX, realY, realWidth, realHeight);
		
	}

	public void setTitle(String string) {
		this.title = string;
		markDirty();
	}
}
