package slgui.swing;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;

import slgui.base.MediaPainter;

public class G2DPainter extends MediaPainter {
	public final Graphics2D g2d;
	int previousPixel;
	double previousRadius = -0.0;
	
	public G2DPainter(Graphics2D g2d) {
		this.g2d = g2d;
		previousPixel = g2d.getColor().getRGB();
		setRadius(1.0);
	}
	
	private final void setPixel(int x) {
		if (x != previousPixel) {
			Color c = new Color(x, true);
			g2d.setColor(c);
			previousPixel = x;
		}
	}
	
	private final void setRadius(double x) {
		if (x != previousPixel) {
			BasicStroke stroke = new BasicStroke((float) x*2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
			g2d.setStroke(stroke);
			previousRadius = x;
		}
	}
	
	@Override
	public void line(double x1, double y1, double x2, double y2, int pixel, double radius) {
		if (radius > 0) {
			x1 -= 0.5;
			y1 -= 0.5;
			x2 -= 0.5;
			y2 -= 0.5;
			setPixel(pixel);
			setRadius(radius);
			g2d.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
		}
	}
	
	@Override
	public void box(double x, double y, double width, double height, int pixel) {
		if (width > 0 && height > 0) {
			x -= 0.5;
			y -= 0.5;
			setPixel(pixel);
			g2d.fillRect((int) x, (int) y, (int) width, (int) height);
		}
	}
}
