package slgui.swing;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Timer;

import javax.swing.JComponent;

import slgui.base.MediaInputState;
import slgui.base.MediaModel;
import slgui.base.MediaPointerButton;
import slgui.base.MediaViewDelegate;

public class GameComponent extends JComponent implements MediaViewDelegate { 
	private MediaModel model;
	private Timer animationTimer;
	private MediaInputState input = new MediaInputState();

	public GameComponent() {
		// TODO Auto-generated constructor stub
	}
	
	public synchronized void stopAnimationTimer() {
		if (animationTimer != null) {
			animationTimer.stop();
			animationTimer = null;
		}
	}
	
	public synchronized void startAnimationTimer(int millis) {
		stopAnimationTimer();
		animationTimer = new Timer(millis, new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				repaint();
			}
		});
		animationTimer.setRepeats(true);
		animationTimer.start();
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		g.setColor(Color.black);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(Color.red);
		if (model == null) {
			g.drawString("No game initialised", getWidth()/2, getHeight()/2);
		} else {
			model.assertScreenSize(getWidth(), getHeight());
			G2DPainter painter = new G2DPainter((Graphics2D) g);
			model.paintView(this, painter);
			g.drawString("TODO...", getWidth()/2, getHeight()/2);
		}
		//super.paintComponent(g);
	}
	
	boolean leftDown = false;

	public void setModel(MediaModel m) {
		if (m == model) {
			return;
		}
		if (model != null) {
			model.detachView(this);
			stopAnimationTimer();
		}
		model = m;
		if (model != null) {
			model.attachView(this);
			if (model.isAnimated()) {
				startAnimationTimer(100);
			}
			if (model.isKeyed()) {
				setFocusable(true);
				requestFocusInWindow();
				addKeyListener(new KeyListener() {
					
					@Override
					public void keyTyped(KeyEvent e) {
						// TODO Auto-generated method stub
						
					}
					
					@Override
					public void keyReleased(KeyEvent e) {
						input.setKeypad(e.getKeyText(e.getKeyCode()), false);
						model.setInputState(input);
					}
					
					@Override
					public void keyPressed(KeyEvent e) {
						input.setKeypad(e.getKeyText(e.getKeyCode()), true);
						model.setInputState(input);
					}
				});
			}
			
			addMouseListener(new MouseListener() {
				
				@Override
				public void mouseReleased(MouseEvent e) {
					if (e.getButton() == MouseEvent.BUTTON1) {
						leftDown = false;
					}
					input.setPointer("mouse", e.getX(), e.getY(), 1).setButton(MediaPointerButton.MAIN, leftDown);
					model.setInputState(input);
				}
				
				@Override
				public void mousePressed(MouseEvent e) {
					if (e.getButton() == MouseEvent.BUTTON1) {
						leftDown = true;
					}
					input.setPointer("mouse", e.getX(), e.getY(), 1).setButton(MediaPointerButton.MAIN, leftDown);
					model.setInputState(input);
				}
				
				@Override
				public void mouseExited(MouseEvent e) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void mouseEntered(MouseEvent e) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void mouseClicked(MouseEvent e) {
					// TODO Auto-generated method stub
					
				}
			});
			
			addMouseMotionListener(new MouseMotionListener() {
				
				@Override
				public void mouseMoved(MouseEvent e) {
					input.setPointer("mouse", e.getX(), e.getY(), 1).setButton(MediaPointerButton.MAIN, leftDown);
					model.setInputState(input);
				}
				
				@Override
				public void mouseDragged(MouseEvent e) {
					input.setPointer("mouse", e.getX(), e.getY(), 1).setButton(MediaPointerButton.MAIN, leftDown);
					model.setInputState(input);
				}
			});
		}
		repaint();
	}
}
