package slgui.themes;

import slgui.base.MediaBox;
import slgui.base.MediaFont;
import slgui.base.MediaInputState;
import slgui.base.MediaPainter;
import slgui.base.MediaPointerButton;
import slgui.gfx.Painter;

public class Theme {

	public Theme() {
		// TODO Auto-generated constructor stub
	}
	
	public MediaFont getFont() {
		return new MediaFont(2);
	}
	
	public int getBorderPixel() {
		return 0xFF808080;
	}
	
	public void paintSpecialButton(MediaPainter p, MediaInputState input, SpecialButtonType t, double x, double y, double width, double height, ButtonState overrideState) {
		if (overrideState == null) {
			overrideState = ButtonState.INACTIVE;
			if (input != null) {
				if (input.downWithin(MediaPointerButton.MAIN, x, y, width, height)) {
					overrideState = ButtonState.ACTIVE;
				} else if (input.hoverWithin(x, y, width, height)) {
					overrideState = ButtonState.HOVER;
				}
			}
		}
		p.box(x, y, width, height, overrideState == ButtonState.ACTIVE ? 0xFFFFC050 : (overrideState == ButtonState.HOVER ? 0xFF50FFC0 : 0xFFFFFFFF));
		p.line(x, y, x + width, y, getBorderPixel(), 1.0);
		p.line(x, y+height, x + width, y+height, getBorderPixel(), 1.0);
		p.line(x, y, x, y + height, getBorderPixel(), 1.0);
		p.line(x + width, y, x + width, y + height, getBorderPixel(), 1.0);
		
		switch (t) {
		case BLANK:
			break;
		case CLOSE:
			p.line(x, y, x+width,y+height, getBorderPixel(), 1.0);
			p.line(x, y+height, x+width,y, getBorderPixel(), 1.0);
			break;
		default:
			p.line(x, y, x+width,y+height, 0xFFFF0000, 1.0);
			p.line(x, y+height, x+width,y, 0xFFFF0000, 1.0);
			break;
		}
	}
	
	public void paintButton(MediaPainter p, MediaInputState input, String label, double x, double y, double width, double height) {
		paintSpecialButton(p, input, SpecialButtonType.BLANK, x, y, width, height, input.keypadDown(label.toUpperCase()) ? ButtonState.ACTIVE : null);
		getFont().render(p, label, x + (width/2) - ((4 + label.length()*16)/2), y + (height/2) - 8, 0xFF0000FF);
	}
	
	public void paintWindowBorder(MediaPainter p, MediaInputState input, String title, double x, double y, double width, double height) {
		int topcolour;
		if (input != null && input.hoverWithin(x, y, width, 24)) {
			topcolour = 0xF0FFC050;
		} else {
			topcolour = 0xE0E0FFE0;
		}
		p.box(x, y, width, 24, topcolour);
		p.line(x, y, x + width, y, getBorderPixel(), 1.0);
		p.line(x, y+24, x + width, y+24, getBorderPixel(), 1.0);
		p.line(x, y+height, x + width, y+height, getBorderPixel(), 1.0);
		p.line(x, y, x, y + height, getBorderPixel(), 1.0);
		p.line(x + width, y, x + width, y + height, getBorderPixel(), 1.0);
		p.line(x + width, y + height - 16, x + width - 16, y + height, getBorderPixel(), 1.0);
		getFont().render(p, title, x + 24, y + 4, 0xFF0000FF);
	}
	
	public void paintDemoWindow(MediaPainter p, MediaInputState input, String title, double x, double y, double width, double height) {
		paintWindowBorder(p, input, title, x, y, width, height);
		paintSpecialButton(p, input, SpecialButtonType.BLANK, x + 4, y + 4, 16, 16, null);
		paintSpecialButton(p, input, SpecialButtonType.CLOSE, x + width - 20, y + 4, 16, 16, null);
	}
	
	public MediaBox desktopContentArea(MediaPainter p, double x, double y, double width, double height, KeypadType keypad, double keypadratio, StatusMode statusMode, PanelMode panelMode) {
		if (keypadratio < 0) {
			keypadratio = 0;
		} else if (keypadratio > 1) {
			keypadratio = 1;
		}
		if (keypad != KeypadType.NONE) {
			double kph = height/3;
			double kpw = 2*width/3;
			double kpx = x + (width/2) - (kpw/2);
			double kpy = y+height-40-(kph+1) + ((1.0-keypadratio)*(kph+100));
			//paintKeypad(p, keypad, kpx, kpy, kpw, kph, 100);
			return new MediaBox(x+0, y+24, width, Math.min(kpy-(y+24), height-(24+40)));
		} else {
			return new MediaBox(x+0, y+24, width, height-(24+40));
		}
	}

	public void paintDesktopControls(MediaPainter p, MediaInputState input, String status1, String status2, double x, double y, double width, double height, KeypadType keypad, double keypadratio, StatusMode statusMode, PanelMode panelMode) {
		p.box(x, y, width, 24, 0xDDFFFFFF);
		//p.line(x, y, x + width, y, getBorderPixel(), 1.0);
		p.line(x, y+24, x + width, y+24, getBorderPixel(), 1.0);
		//p.line(x, y+height, x + width, y+height, getBorderPixel(), 1.0);
		//p.line(x, y, x, y + height, getBorderPixel(), 1.0);
		for (int i = 0; i*2 < 40; i++) {
			p.box(x, y+height-(40-i*2), width, 42-i*2, 0x40000000);//0x40FFFFFF);
		}
		p.line(x, y+height-40, x + width, y+height-40, getBorderPixel(), 1.0);
		//p.line(x + width, y, x + width, y + height, getBorderPixel(), 1.0);
		getFont().render(p, status1, x + 24, y + 4, 0xFF0000FF);
		getFont().render(p, status2, x + width - (4 + status2.length()*16), y + 4, 0xFF0000FF);
	}
	
	public void paintDemoDesktop(MediaPainter p, MediaInputState input, String status1, String status2, double x, double y, double width, double height, KeypadType keypad, double keypadratio, StatusMode statusMode, PanelMode panelMode) {
		if (keypadratio < 0) {
			keypadratio = 0;
		} else if (keypadratio > 1) {
			keypadratio = 1;
		}
		if (keypad != KeypadType.NONE) {
			double kph = height/3;
			double kpw = 2*width/3;
			double kpx = x + (width/2) - (kpw/2);
			double kpy = y+height-40-(kph+1) + ((1.0-keypadratio)*(kph+100));
			paintKeypad(p, input, keypad, kpx, kpy, kpw, kph, 100);
		}
		paintDesktopControls(p, input, status1, status2, x, y, width, height, keypad, keypadratio, statusMode, panelMode);
		paintSpecialButton(p, input, SpecialButtonType.BLANK, x + 4, y + 4, 16, 16, null);
		paintSpecialButton(p, input, SpecialButtonType.HOME, x + (width/2) - 16, y + height - 36, 32, 32, null);
		paintSpecialButton(p, input, SpecialButtonType.HOME, x + (width/2) - 56, y + height - 36, 32, 32, null);
		paintSpecialButton(p, input, SpecialButtonType.HOME, x + (width/2) + 24, y + height - 36, 32, 32, null);
	}
	
	public void paintKeypad(MediaPainter p, MediaInputState input, KeypadType t, double x, double y, double width, double height, double xheight) {
		p.box(x, y, width, height+xheight, 0xDDFFFFFF);
		p.line(x, y, x + width, y, getBorderPixel(), 1.0);
		p.line(x, y+height+xheight, x + width, y+height+xheight, getBorderPixel(), 1.0);
		p.line(x, y, x, y + height+xheight, getBorderPixel(), 1.0);
		p.line(x + width, y, x + width, y + height+xheight, getBorderPixel(), 1.0);
		
		switch(t) {
		case NUMBERS:
			paintNumpad(p, input, x, y, width, height);
			break;
		case LATIN_UPPER:
			paintLatinKeypad(p, input, x, y, width, height, true);
			break;
		case LATIN_LOWER:
			paintLatinKeypad(p, input, x, y, width, height, false);
			break;
		}
	}

	public void paintNumpad(MediaPainter p, MediaInputState input, double x, double y, double width, double height) {
		double spc = (width+height)/50;
		double w = (width - spc*4)/3;
		double h = (height - spc*5)/4;

		paintButton(p, input, "1", x+spc, y+spc, w, h);
		paintButton(p, input, "2", x+spc+w+spc, y+spc, w, h);
		paintButton(p, input, "3", x+spc+(w+spc)*2, y+spc, w, h);

		paintButton(p, input, "4", x+spc, y+spc+(h+spc), w, h);
		paintButton(p, input, "5", x+spc+w+spc, y+spc+(h+spc), w, h);
		paintButton(p, input, "6", x+spc+(w+spc)*2, y+spc+(h+spc), w, h);

		paintButton(p, input, "7", x+spc, y+spc+(h+spc)*2, w, h);
		paintButton(p, input, "8", x+spc+w+spc, y+spc+(h+spc)*2, w, h);
		paintButton(p, input, "9", x+spc+(w+spc)*2, y+spc+(h+spc)*2, w, h);

		//paintButton(p, "7", x+spc, y+spc+(h+spc)*3, w, h);
		paintButton(p, input, "0", x+spc+w+spc, y+spc+(h+spc)*3, w, h);
		//paintButton(p, "9", x+spc+(w+spc)*2, y+spc+(h+spc)*3, w, h);
	}
	
	public String recase(String input, boolean upcase) {
		if (upcase) {
			return input.toUpperCase();
		} else {
			return input.toLowerCase();
		}
	}
	
	public void paintKeypadRow(MediaPainter p, MediaInputState input, double x, double y, double width, double height, boolean upcase, double spc, double bw, double bh, String...strings) {
		x = (x + width/2) - ((strings.length*bw + (strings.length-1)*spc)/2);
		for (int i = 0; i < strings.length; i++) {
			String s = recase(strings[i], upcase);
			double bx = x + (i * (bw + spc));
			double by = y + spc;
			paintButton(p, input, s, bx, by, bw, bh);
		}
	}

	public void paintLatinKeypad(MediaPainter p, MediaInputState input, double x, double y, double width, double height, boolean upcase) {
		double spc = 8;//(width+height)/50;
		double w = (width - spc*11)/10;
		double h = (height - spc*5)/4;

		paintKeypadRow(p, input, x, y, width, height, upcase, spc, w, h, "q", "w", "e", "r", "t", "y", "u", "i", "o", "p");
		paintKeypadRow(p, input, x, y+(h+spc), width, height, upcase, spc, w, h, "a", "s", "d", "f", "g", "h", "j", "k", "l");
		paintKeypadRow(p, input, x, y+(h+spc)*2, width, height, upcase, spc, w, h, "z", "x", "c", "v", "b", "n", "m");
		
		double spcw = w*3+spc*2;
		paintButton(p, input, " ", x + width/2 - spcw/2, y+spc+(h+spc)*3, spcw, h);

		/*paintButton(p, "1", x+spc, y+spc, w, h);
		paintButton(p, "2", x+spc+w+spc, y+spc, w, h);
		paintButton(p, "3", x+spc+(w+spc)*2, y+spc, w, h);

		paintButton(p, "4", x+spc, y+spc+(h+spc), w, h);
		paintButton(p, "5", x+spc+w+spc, y+spc+(h+spc), w, h);
		paintButton(p, "6", x+spc+(w+spc)*2, y+spc+(h+spc), w, h);

		paintButton(p, "7", x+spc, y+spc+(h+spc)*2, w, h);
		paintButton(p, "8", x+spc+w+spc, y+spc+(h+spc)*2, w, h);
		paintButton(p, "9", x+spc+(w+spc)*2, y+spc+(h+spc)*2, w, h);

		//paintButton(p, "7", x+spc, y+spc+(h+spc)*3, w, h);
		paintButton(p, "0", x+spc+w+spc, y+spc+(h+spc)*3, w, h);
		//paintButton(p, "9", x+spc+(w+spc)*2, y+spc+(h+spc)*3, w, h);
		 */
	}
}
